/**
 * @author Dave Toth
 * @date Summer and Fall 2006
 * 
 * This policy dictates the client's behavior if the client uses the buffer 
 * none policy for downloading workunits.
 */

package edu.wpi.cs.toth.task_distribution_simulator;

import java.util.Hashtable;

class BufferMultiplePolicy extends DownloadPolicy{

    ///////////////////////////////////////////////////////////////////////////
    // Constants
    ///////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    // Variables
    ///////////////////////////////////////////////////////////////////////////

    /**
     * This Hashtable holds the percent of time the cpu is available for 
     * volunteer computing for each of the traces.
     */
    private Hashtable cpuAvailabilityTable;


    ///////////////////////////////////////////////////////////////////////////
    // Methods
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Constructor.  Fill the Hashtable with the percent of the time each
     * computer is available for volunteer computing.
     */
    public BufferMultiplePolicy() {
	cpuAvailabilityTable = new Hashtable();
	// Debugging
	cpuAvailabilityTable.put("testtrace_buffer_many_test", new Float(72));
	// End Debugging

	cpuAvailabilityTable.put("business_trace_1.trc", new Float(1.96));
	cpuAvailabilityTable.put("business_trace_10.trc", new Float(75.97));
	cpuAvailabilityTable.put("business_trace_11.trc", new Float(4.02));
	cpuAvailabilityTable.put("business_trace_12.trc", new Float(8.77));
	cpuAvailabilityTable.put("business_trace_13.trc", new Float(10.43));
	cpuAvailabilityTable.put("business_trace_14.trc", new Float(5.39));
	cpuAvailabilityTable.put("business_trace_15.trc", new Float(4.06));
	cpuAvailabilityTable.put("business_trace_16.trc", new Float(53.22));
	cpuAvailabilityTable.put("business_trace_17.trc", new Float(51.68));
	cpuAvailabilityTable.put("business_trace_18.trc", new Float(17.97));
	cpuAvailabilityTable.put("business_trace_19.trc", new Float(70.51));
	cpuAvailabilityTable.put("business_trace_2.trc", new Float(76.96));
	cpuAvailabilityTable.put("business_trace_20.trc", new Float(.67));
	cpuAvailabilityTable.put("business_trace_21.trc", new Float(81.06));
	cpuAvailabilityTable.put("business_trace_22.trc", new Float(51.98));
	cpuAvailabilityTable.put("business_trace_23.trc", new Float(8.63));
	cpuAvailabilityTable.put("business_trace_24.trc", new Float(21.22));
	cpuAvailabilityTable.put("business_trace_25.trc", new Float(3.39));
	cpuAvailabilityTable.put("business_trace_26.trc", new Float(60.69));
	cpuAvailabilityTable.put("business_trace_3.trc", new Float(7.31));
	cpuAvailabilityTable.put("business_trace_4.trc", new Float(9.36));
	cpuAvailabilityTable.put("business_trace_5.trc", new Float(37.46));
	cpuAvailabilityTable.put("business_trace_6.trc", new Float(8.26));
	cpuAvailabilityTable.put("business_trace_7.trc", new Float(3.33));
	cpuAvailabilityTable.put("business_trace_8.trc", new Float(11.15));
	cpuAvailabilityTable.put("business_trace_9.trc", new Float(44.21));
	cpuAvailabilityTable.put("ccc_trace_1.trc", new Float(70.12));
	cpuAvailabilityTable.put("ccc_trace_10.trc", new Float(58.78));
	cpuAvailabilityTable.put("ccc_trace_11.trc", new Float(64.08));
	cpuAvailabilityTable.put("ccc_trace_12.trc", new Float(61.81));
	cpuAvailabilityTable.put("ccc_trace_13.trc", new Float(66.11));
	cpuAvailabilityTable.put("ccc_trace_14.trc", new Float(61.23));
	cpuAvailabilityTable.put("ccc_trace_15.trc", new Float(65.65));
	cpuAvailabilityTable.put("ccc_trace_16.trc", new Float(66.32));
	cpuAvailabilityTable.put("ccc_trace_17.trc", new Float(63.13));
	cpuAvailabilityTable.put("ccc_trace_18.trc", new Float(62.56));
	cpuAvailabilityTable.put("ccc_trace_19.trc", new Float(67.45));
	cpuAvailabilityTable.put("ccc_trace_2.trc", new Float(68.17));
	cpuAvailabilityTable.put("ccc_trace_20.trc", new Float(65.69));
	cpuAvailabilityTable.put("ccc_trace_21.trc", new Float(73.20));
	cpuAvailabilityTable.put("ccc_trace_22.trc", new Float(74.31));
	cpuAvailabilityTable.put("ccc_trace_23.trc", new Float(77.92));
	cpuAvailabilityTable.put("ccc_trace_24.trc", new Float(71.06));
	cpuAvailabilityTable.put("ccc_trace_25.trc", new Float(71.14));
	cpuAvailabilityTable.put("ccc_trace_26.trc", new Float(71.96));
	cpuAvailabilityTable.put("ccc_trace_27.trc", new Float(84.29));
	cpuAvailabilityTable.put("ccc_trace_28.trc", new Float(75.99));
	cpuAvailabilityTable.put("ccc_trace_29.trc", new Float(66.95));
	cpuAvailabilityTable.put("ccc_trace_3.trc", new Float(65.59));
	cpuAvailabilityTable.put("ccc_trace_30.trc", new Float(68.15));
	cpuAvailabilityTable.put("ccc_trace_31.trc", new Float(75.90));
	cpuAvailabilityTable.put("ccc_trace_32.trc", new Float(81.54));
	cpuAvailabilityTable.put("ccc_trace_33.trc", new Float(80.75));
	cpuAvailabilityTable.put("ccc_trace_34.trc", new Float(65.05));
	cpuAvailabilityTable.put("ccc_trace_35.trc", new Float(69.43));
	cpuAvailabilityTable.put("ccc_trace_36.trc", new Float(67.18));
	cpuAvailabilityTable.put("ccc_trace_37.trc", new Float(70.40));
	cpuAvailabilityTable.put("ccc_trace_38.trc", new Float(62.69));
	cpuAvailabilityTable.put("ccc_trace_39.trc", new Float(68.29));
	cpuAvailabilityTable.put("ccc_trace_4.trc", new Float(63.83));
	cpuAvailabilityTable.put("ccc_trace_40.trc", new Float(67.77));
	cpuAvailabilityTable.put("ccc_trace_41.trc", new Float(75.62));
	cpuAvailabilityTable.put("ccc_trace_42.trc", new Float(63.15));
	cpuAvailabilityTable.put("ccc_trace_43.trc", new Float(89.18));
	cpuAvailabilityTable.put("ccc_trace_44.trc", new Float(79.19));
	cpuAvailabilityTable.put("ccc_trace_45.trc", new Float(91.91));
	cpuAvailabilityTable.put("ccc_trace_46.trc", new Float(82.68));
	cpuAvailabilityTable.put("ccc_trace_47.trc", new Float(84.43));
	cpuAvailabilityTable.put("ccc_trace_48.trc", new Float(78.81));
	cpuAvailabilityTable.put("ccc_trace_49.trc", new Float(82.42));
	cpuAvailabilityTable.put("ccc_trace_5.trc", new Float(66.52));
	cpuAvailabilityTable.put("ccc_trace_50.trc", new Float(88.03));
	cpuAvailabilityTable.put("ccc_trace_51.trc", new Float(82.53));
	cpuAvailabilityTable.put("ccc_trace_52.trc", new Float(88.97));
	cpuAvailabilityTable.put("ccc_trace_53.trc", new Float(83.86));
	cpuAvailabilityTable.put("ccc_trace_54.trc", new Float(79.55));
	cpuAvailabilityTable.put("ccc_trace_55.trc", new Float(75.12));
	cpuAvailabilityTable.put("ccc_trace_56.trc", new Float(88.15));
	cpuAvailabilityTable.put("ccc_trace_57.trc", new Float(73.08));
	cpuAvailabilityTable.put("ccc_trace_58.trc", new Float(91.54));
	cpuAvailabilityTable.put("ccc_trace_59.trc", new Float(89.41));
	cpuAvailabilityTable.put("ccc_trace_6.trc", new Float(73.31));
	cpuAvailabilityTable.put("ccc_trace_60.trc", new Float(81.64));
	cpuAvailabilityTable.put("ccc_trace_61.trc", new Float(82.98));
	cpuAvailabilityTable.put("ccc_trace_62.trc", new Float(85.34));
	cpuAvailabilityTable.put("ccc_trace_63.trc", new Float(84.29));
	cpuAvailabilityTable.put("ccc_trace_64.trc", new Float(80.12));
	cpuAvailabilityTable.put("ccc_trace_65.trc", new Float(80.58));
	cpuAvailabilityTable.put("ccc_trace_66.trc", new Float(81.99));
	cpuAvailabilityTable.put("ccc_trace_67.trc", new Float(92.53));
	cpuAvailabilityTable.put("ccc_trace_68.trc", new Float(84.04));
	cpuAvailabilityTable.put("ccc_trace_7.trc", new Float(58.40));
	cpuAvailabilityTable.put("ccc_trace_8.trc", new Float(61.91));
	cpuAvailabilityTable.put("ccc_trace_9.trc", new Float(62.12));
	cpuAvailabilityTable.put("home_trace_1.trc", new Float(36.68));
	cpuAvailabilityTable.put("home_trace_10.trc", new Float(69.85));
	cpuAvailabilityTable.put("home_trace_12.trc", new Float(.15));
	cpuAvailabilityTable.put("home_trace_13.trc", new Float(48.21));
	cpuAvailabilityTable.put("home_trace_14.trc", new Float(50.32));
	cpuAvailabilityTable.put("home_trace_15.trc", new Float(1.18));
	cpuAvailabilityTable.put("home_trace_16.trc", new Float(36.16));
	cpuAvailabilityTable.put("home_trace_17.trc", new Float(.69));
	cpuAvailabilityTable.put("home_trace_18.trc", new Float(1.04));
	cpuAvailabilityTable.put("home_trace_2.trc", new Float(2.91));
	cpuAvailabilityTable.put("home_trace_21.trc", new Float(36.88));
	cpuAvailabilityTable.put("home_trace_22.trc", new Float(2.95));
	cpuAvailabilityTable.put("home_trace_23.trc", new Float(45.51));
	cpuAvailabilityTable.put("home_trace_24.trc", new Float(38.33));
	cpuAvailabilityTable.put("home_trace_25.trc", new Float(87.12));
	cpuAvailabilityTable.put("home_trace_3.trc", new Float(17.48));
	cpuAvailabilityTable.put("home_trace_4.trc", new Float(75.59));
	cpuAvailabilityTable.put("home_trace_5.trc", new Float(75.28));
	cpuAvailabilityTable.put("home_trace_6.trc", new Float(37.18));
	cpuAvailabilityTable.put("home_trace_7.trc", new Float(10.05));
	cpuAvailabilityTable.put("home_trace_9.trc", new Float(4.96));
	cpuAvailabilityTable.put("student_10.trc", new Float(5.29));
	cpuAvailabilityTable.put("student_11.trc", new Float(13.53));
	cpuAvailabilityTable.put("student_12.trc", new Float(49.99));
	cpuAvailabilityTable.put("student_14.trc", new Float(80.95));
	cpuAvailabilityTable.put("student_15.trc", new Float(59.43));
	cpuAvailabilityTable.put("student_16.trc", new Float(36.25));
	cpuAvailabilityTable.put("student_17.trc", new Float(85.38));
	cpuAvailabilityTable.put("student_19.trc", new Float(51.19));
	cpuAvailabilityTable.put("student_2.trc", new Float(3.82));
	cpuAvailabilityTable.put("student_20.trc", new Float(62.82));
	cpuAvailabilityTable.put("student_21.trc", new Float(.05));
	cpuAvailabilityTable.put("student_22.trc", new Float(32.90));
	cpuAvailabilityTable.put("student_23.trc", new Float(7.21));
	cpuAvailabilityTable.put("student_26.trc", new Float(.03));
	cpuAvailabilityTable.put("student_27.trc", new Float(8.41));
	cpuAvailabilityTable.put("student_30.trc", new Float(7.74));
	cpuAvailabilityTable.put("student_31.trc", new Float(59.43));
	cpuAvailabilityTable.put("student_32.trc", new Float(6.93));
	cpuAvailabilityTable.put("student_33.trc", new Float(.61));
	cpuAvailabilityTable.put("student_34.trc", new Float(2.65));
	cpuAvailabilityTable.put("student_36.trc", new Float(13.88));
	cpuAvailabilityTable.put("student_38.trc", new Float(1.47));
	cpuAvailabilityTable.put("student_4.trc", new Float(47.24));
	cpuAvailabilityTable.put("student_5.trc", new Float(68.55));
	cpuAvailabilityTable.put("student_7.trc", new Float(12.19));
    }
    

    /**
     * Returns how many workunits need to be downloaded.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return - Returns how many workunits need to be downloaded.
     */
    int howManyWorkunitsToDownload(SimulatedClient client) {
	float cpuPercentAvailable = getPercentCpuAvailable(client.getTraceFileName());
	//System.out.println("cpu avail: " + cpuPercentAvailable);
	float timeAvailableToProcess = (cpuPercentAvailable/100) * ((float) client.getDelayBound());
	//System.out.println("time avail: " + timeAvailableToProcess);
	int timeAvailable = (int) timeAvailableToProcess;
	//System.out.println("time avail: " + timeAvailableToProcess);
	timeAvailable = timeAvailable - (client.getTaskDuration() * client.getNumberOfBufferedTasks());
	//System.out.println("time avail: " + timeAvailableToProcess);
	if (client.getCurrentWorkunit() != null) {
	    timeAvailable = timeAvailable - client.getTaskDuration();
	}
	//System.out.println("time avail: " + timeAvailableToProcess);
	int maxTasks = (int) (timeAvailable/client.getTaskDuration());
	//System.out.println("max tasks: " + maxTasks);

	// this.tasksToBuffer = Math.max(1, (int) ((daysOfTasksToBuffer * 86400)/this.taskDuration));


	// If we're in the process of refilling the buffer...
	if (client.getDownloadStatus() == SimulatedClient.REFILL) {
	    if (client.getNumberOfBufferedTasks() > client.getNumberOfTasksToBuffer()) {
		//System.out.println("$$$$$ REFILL DOWNLOADS LEFT: " + 0);
		return 0;
	    }
	    else {
		//System.out.println("$$$$$ REFILL DOWNLOADS LEFT: " + (tasksToBuffer - tasksAlreadyInBuffer + 1));
		return Math.min(maxTasks, client.getNumberOfTasksToBuffer() - client.getNumberOfBufferedTasks() + client.isFirstTimeFillingBuffer());
	    }
	}
	// Otherwise, we're checking to see if we need to refill
	else {
	    if (client.getNumberOfBufferedTasks() >= 1) {
		//System.out.println("$$$$$ query DOWNLOADS LEFT: " + 0);
		return 0;
	    }
	    else {
		//System.out.println("$$$$$ query DOWNLOADS LEFT: " + (tasksToBuffer - tasksAlreadyInBuffer) + 1));
		return Math.min(maxTasks, client.getNumberOfTasksToBuffer() - client.getNumberOfBufferedTasks() + client.isFirstTimeFillingBuffer());
	    }
	}
    }
  
    /**
     * Returns whether the client needs to download another workunit 
     * immediately.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return  - Returns whether the client needs to download another 
     *            workunit immediately.
     */
    boolean needToDownload(SimulatedClient client) {
	if (client.getNumberOfBufferedTasks() < 1) { 
	    float cpuPercentAvailable = getPercentCpuAvailable(client.getTraceFileName());
	    //System.out.println("cpu avail: " + cpuPercentAvailable);
	    float timeAvailableToProcess = (cpuPercentAvailable/100) * ((float) client.getDelayBound());
	    //System.out.println("time avail: " + timeAvailableToProcess);
	    int timeAvailable = (int) timeAvailableToProcess;
	    //System.out.println("time avail: " + timeAvailableToProcess);
	    timeAvailable = timeAvailable - (client.getTaskDuration() * client.getNumberOfBufferedTasks());
	    //System.out.println("time avail: " + timeAvailableToProcess);
	    if (client.getCurrentWorkunit() != null) {
		timeAvailable = timeAvailable - client.getTaskDuration();
	    }
	    //System.out.println("time avail: " + timeAvailableToProcess);
	    int maxTasks = (int) (timeAvailable/client.getTaskDuration());
	    //System.out.println("max tasks: " + maxTasks);
	    if (maxTasks > 0) {
		return true;
	    }
	    else {
		return false;
	    }
	}
	else {
	    return false;
	}
    }

    /**
     * Returns whether the client needs to download an additional task to
     * fill the buffer to the correct state.  This occurs on the the first
     * time the client tries to fill the buffer for certain buffering methods.
     * In particular, Buffer None and Buffer None Dwonload Early never need
     * to do this, but Buffer 1 and Buffer Multiple do need to do this.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return - Returns whether the client needs to download one additional
     *           task to fill the buffer correctly for the given policy.
     */
    boolean needToDownloadAdditionalTask(SimulatedClient client) {
	if (client.isFirstTimeFillingBuffer() == 1) {
	    return true;
	}
	else {
	    return false;
	}
    }

    /**
     * Returns the time required to download a workunit.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return Returns the time required to download a workunit.
     */
    int getDownloadTime(SimulatedClient client) {
	int fileSize = client.getFileSize();
	int downloadSpeed = client.getDownloadSpeed();
	if (fileSize % downloadSpeed != 0) {
	    return (int) (fileSize/downloadSpeed) + 1;
	}
	else {
	    return (int) (fileSize/downloadSpeed);
	}
	//	return 0;
    }   

    /**
     * Returns whether a task is pipelined.  Pipelined tasks do not
     * take the time to download them away from the client.  They are
     * assumed to be downloaded as the CPU is working on its current
     * workunit because the effect on the CPU from downloading is 
     * minimal.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return Returns whether a task is pipelined.
     */
    boolean isTaskPipelined(SimulatedClient client) {
	if ((client.getCurrentWorkunit() != null) || (client.getNumberOfBufferedTasks() != 0)) {
	    return true;
	}
	else {
	    return false;
	}
    }

    /**
     * Sets the state of the download flag with true indicating a download
     * has been initiated but not completed.
     * @param state - The new state of the flag.
     */
    void setDownloadFlag(boolean state) {
	downloadFlag = state;
    }

    /**
     * Returns the state of the download flag with true indicating a download
     * has been initiated but not completed.
     * @return state - Returns the state of the flag.
     */
    boolean getDownloadFlag() {
	return downloadFlag;
    }

    /**
     * Returns the percent of cpu time (as a float) that a computer is avaiable
     * for volunteer computing.
     * @return - Returns the percent of cpu time (as a float) that a computer
     *           is avaiable for volunteer computing.
     */
    float getPercentCpuAvailable(String trace) {
	Float val = (Float) cpuAvailabilityTable.get(trace);
	float f = val.floatValue();
	return f;
    }
}
