/**
 * @author Dave Toth
 * @date Summer and Fall 2006
 * 
 * This policy dictates the client's behavior if the client uses the buffer 
 * none policy for downloading workunits.
 */

package edu.wpi.cs.toth.task_distribution_simulator;

class BufferNonePolicy extends DownloadPolicy{

    ///////////////////////////////////////////////////////////////////////////
    // Constants
    ///////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    // Variables
    ///////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////
    // Methods
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Constructor.  
     */
    public BufferNonePolicy() {
    }
    

    /**
     * Returns how many workunits need to be downloaded.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return - Returns how many workunits need to be downloaded.
     */
    public int howManyWorkunitsToDownload(SimulatedClient client) {
	if (client.getCurrentWorkunit() != null) {
	    return 0;
	}
	else {
	    return 1;
	}
    }

    /**
     * Returns whether the client needs to download another workunit 
     * immediately.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return  - Returns whether the client needs to download another 
     *            workunit immediately.
     */
    boolean needToDownload(SimulatedClient client) {
	if ((client.getCurrentWorkunit() == null) && (client.getNumberOfBufferedTasks() < 1)) {
	    return true;
	}
	else {
	    return false;
	}
    }

    /**
     * Returns whether the client needs to download an additional task to
     * fill the buffer to the correct state.  This occurs on the the first
     * time the client tries to fill the buffer for certain buffering methods.
     * In particular, Buffer None and Buffer None Dwonload Early never need
     * to do this, but Buffer 1 and Buffer Multiple do need to do this.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return - Returns whether the client needs to download one additional
     *           task to fill the buffer correctly for the given policy.
     */
    boolean needToDownloadAdditionalTask(SimulatedClient client) {
	return false;
    }

    /**
     * Returns the time required to download a workunit.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return Returns the time required to download a workunit.
     */
    int getDownloadTime(SimulatedClient client) {
	int fileSize = client.getFileSize();
	int downloadSpeed = client.getDownloadSpeed();
	if (fileSize % downloadSpeed != 0) {
	    return (int) (fileSize/downloadSpeed) + 1;
	}
	else {
	    return (int) (fileSize/downloadSpeed);
	}
    }   

    /**
     * Returns whether a task is pipelined.  Pipelined tasks do not
     * take the time to download them away from the client.  They are
     * assumed to be downloaded as the CPU is working on its current
     * workunit because the effect on the CPU from downloading is 
     * minimal.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return Returns whether a task is pipelined.
     */
    boolean isTaskPipelined(SimulatedClient client) {
	return false;
    }

    /**
     * Sets the state of the download flag with true indicating a download
     * has been initiated but not completed.
     * @param state - The new state of the flag.
     */
    void setDownloadFlag(boolean state) {
	downloadFlag = state;
    }

    /**
     * Returns the state of the download flag with true indicating a download
     * has been initiated but not completed.
     * @return state - Returns the state of the flag.
     */
    boolean getDownloadFlag() {
	return downloadFlag;
    }
}
