/**
 * @author Dave Toth
 * @date Summer and Fall 2006
 * 
 * This abstract class provides the template that the actual download policies
 * must implement.
 */

package edu.wpi.cs.toth.task_distribution_simulator;

abstract class DownloadPolicy {

    ///////////////////////////////////////////////////////////////////////////
    // Constants
    ///////////////////////////////////////////////////////////////////////////


    ///////////////////////////////////////////////////////////////////////////
    // Variables
    ///////////////////////////////////////////////////////////////////////////

    /**
     * This flag indicates that a download was initiated but completed if it is
     * true.  It is only relevant for the BufferNoneDownloadEarly policy.
     */
    boolean downloadFlag;

    ///////////////////////////////////////////////////////////////////////////
    // Methods
    ///////////////////////////////////////////////////////////////////////////


    /**
     * Constructor.
     */
    DownloadPolicy() {
    }



    /**
     * Returns how many workunits need to be downloaded.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return - Returns how many workunits need to be downloaded.
     */
    abstract int howManyWorkunitsToDownload(SimulatedClient client);

    /**
     * Returns whether the client needs to download another workunit 
     * immediately.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return  - Returns whether the client needs to download another 
     *            workunit immediately.
     */
    abstract boolean needToDownload(SimulatedClient client);

    /**
     * Returns whether the client needs to download an additional task to
     * fill the buffer to the correct state.  This occurs on the the first
     * time the client tries to fill the buffer for certain buffering methods.
     * In particular, Buffer None and Buffer None Dwonload Early never need
     * to do this, but Buffer 1 and Buffer Multiple do need to do this.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return - Returns whether the client needs to download one additional
     *           task to fill the buffer correctly for the given policy.
     */
    abstract boolean needToDownloadAdditionalTask(SimulatedClient client);

    /**
     * Returns the time required to download a workunit.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return Returns the time required to download a workunit.
     */
    abstract int getDownloadTime(SimulatedClient client);

    /**
     * Returns whether a task is pipelined.  Pipelined tasks do not
     * take the time to download them away from the client.  They are
     * assumed to be downloaded as the CPU is working on its current
     * workunit because the effect on the CPU from downloading is 
     * minimal.
     * @param client - A reference to the client.  This allows us to get the
     *                 information we need to do calculations without having
     *                 to modify every policy if we want to add a new policy.
     * @return Returns whether a task is pipelined.
     */
    abstract boolean isTaskPipelined(SimulatedClient client);

    /**
     * Sets the state of the download flag with true indicating a download
     * has been initiated but not completed.
     * @param state - The new state of the flag.
     */
    abstract void setDownloadFlag(boolean state);

    /**
     * Returns the state of the download flag with true indicating a download
     * has been initiated but not completed.
     * @return state - Returns the state of the flag.
     */
    abstract boolean getDownloadFlag();
}
