/**
 * @author Dave Toth
 * @date Summer and Fall 2006
 * 
 * This program simulates a volunteer computing client.  It takes a trace of
 * a computer's available, unavailable but on, and unavailable times and
 * determines how many work units the computer would have completed in the
 * time covered by the trace.
 */

package edu.wpi.cs.toth.task_distribution_simulator;

public class Simulator {

    ///////////////////////////////////////////////////////////////////////////
    // Constants
    ///////////////////////////////////////////////////////////////////////////

    // Buffering methods.
    public static final String bufferingMethod1 = "BUFFER_NOTHING";
    public static final String bufferingMethod2 = "BUFFER_ONE_TASK";
    public static final String bufferingMethod3 = "BUFFER_MULTIPLE_TASKS";
    public static final String bufferingMethod4 = "BUFFER_NOTHING_DOWNLOAD_EARLY";


    ///////////////////////////////////////////////////////////////////////////
    // Variables
    ///////////////////////////////////////////////////////////////////////////


    ///////////////////////////////////////////////////////////////////////////
    // Methods
    ///////////////////////////////////////////////////////////////////////////

    /**
     * The main program.  Creates an instance of the simulator using the 
     * command line arguments and starts the simulation.
     */
    public static void main(String[] args){
	if (args.length < 7) {
	    System.out.println("There were only " + args.length + " args.  They were");
	    for (int index = 0; index < args.length; index ++) {
		System.out.println(args[index]);
	    }
	    printErrorMessage();
	}
	   
	String dataFile = args[0];

	String bufferingMethod = (args[1]);
	if ((bufferingMethod.equals(bufferingMethod1) == false) && 
	    (bufferingMethod.equals(bufferingMethod2) == false) &&
	    (bufferingMethod.equals(bufferingMethod3) == false) &&
	    (bufferingMethod.equals(bufferingMethod4) == false)) {
	    System.out.println("The command line argument specifying the buffering method is invalid.");
	    printErrorMessage();
	}

	float tasksToBuffer = 0;
	try {
	    tasksToBuffer = Float.parseFloat(args[2]);
	}
	catch (Exception e) {
	    System.out.println("The command line argument for the number of days of tasks to buffer was invalid.");
	    printErrorMessage();
	}

	int delayBound = 0;
	try {
	    delayBound = Integer.parseInt(args[3]);
	}
	catch (Exception e) {
	    System.out.println("The command line argument for the delayBound was invalid.");
	    printErrorMessage();
	}

	int taskDuration = 0;
	try {
	    taskDuration = Integer.parseInt(args[4]);
	}
	catch (Exception e) {
	    System.out.println("The command line argument for the task duration was invalid.");
	    printErrorMessage();
	}

	int downloadSpeed = 0;
	try {
	    downloadSpeed = Integer.parseInt(args[5]);
	}
	catch (Exception e) {
	    System.out.println("The command line argument for the download speed was invalid.");
	    printErrorMessage();
	}

	int fileSize = 0;
	try {
	    fileSize = Integer.parseInt(args[6]);
	}
	catch (Exception e) {
	    System.out.println("The command line argument for the file size was invalid.");
	    printErrorMessage();
	}

	// Create an instance of the download policy the client will use.
	DownloadPolicy policy;
	if (bufferingMethod.equals(bufferingMethod1)) {
	    policy = new BufferNonePolicy();
	}
	else if (bufferingMethod.equals(bufferingMethod2)) {
	    policy = new BufferOnePolicy();
	}
	else if (bufferingMethod.equals(bufferingMethod3)) {
	    policy = new BufferMultiplePolicy();
	}
	else {
	    policy = new BufferNoneDownloadEarlyPolicy();
	}	
	
	// Create a "client" and pass control to it.
	SimulatedClient client = new SimulatedClient(dataFile, bufferingMethod, tasksToBuffer, delayBound, taskDuration, downloadSpeed, fileSize, policy);

	// The program is over.  Exit.
	System.exit(0);
    }


    /**
     * Prints an error message telling the user how to run the Simulator and
     * terminates the program.
     */
    public static void printErrorMessage() {
	System.out.println("\n\n");
	System.out.println("To run this program, enter the following command line arguments:\n");
	System.out.println("1. Data file to read trace from.");
	System.out.println("2. Buffering method.");
	System.out.println("3. The number of days of tasks to buffer.");
	System.out.println("4. Delay Bound.");
	System.out.println("5. Task duration (the number of seconds).");
	System.out.println("6. Download speed (the number of bits per second).");
	System.out.println("7. File size (the number of bits in the file).");
	System.out.println("\n");
	System.exit(0);
    }
}

